@@ -242,19 +242,19 @@ void query_subscribers(FILE *rsp)
 	fprintf(rsp, "]");
 }
 
-int query_node_ids(coordinates_t *ref, coordinates_t *trg, monitor_select_t *mon_sel, desktop_select_t *desk_sel, node_select_t *sel, FILE *rsp)
+int query_node_ids(coordinates_t *mon_ref, coordinates_t *desk_ref, coordinates_t* ref, coordinates_t *trg, monitor_select_t *mon_sel, desktop_select_t *desk_sel, node_select_t *sel, FILE *rsp)
 {
 	int count = 0;
 	for (monitor_t *m = mon_head; m != NULL; m = m->next) {
 		coordinates_t loc = {m, NULL, NULL};
 		if ((trg->monitor != NULL && m != trg->monitor) ||
-		    (mon_sel != NULL && !monitor_matches(&loc, ref, mon_sel))) {
+		    (mon_sel != NULL && !monitor_matches(&loc, mon_ref, mon_sel))) {
 			continue;
 		}
 		for (desktop_t *d = m->desk_head; d != NULL; d = d->next) {
 			coordinates_t loc = {m, d, NULL};
 			if ((trg->desktop != NULL && d != trg->desktop) ||
-			    (desk_sel != NULL && !desktop_matches(&loc, ref, desk_sel))) {
+			    (desk_sel != NULL && !desktop_matches(&loc, desk_ref, desk_sel))) {
 				continue;
 			}
 			count += query_node_ids_in(d->root, d, m, ref, trg, sel, rsp);
@@ -281,13 +281,13 @@ int query_node_ids_in(node_t *n, desktop_t *d, monitor_t *m, coordinates_t *ref,
 	return count;
 }
 
-int query_desktop_ids(coordinates_t *ref, coordinates_t *trg, monitor_select_t *mon_sel, desktop_select_t *sel, desktop_printer_t printer, FILE *rsp)
+int query_desktop_ids(coordinates_t* mon_ref, coordinates_t *ref, coordinates_t *trg, monitor_select_t *mon_sel, desktop_select_t *sel, desktop_printer_t printer, FILE *rsp)
 {
 	int count = 0;
 	for (monitor_t *m = mon_head; m != NULL; m = m->next) {
 		coordinates_t loc = {m, NULL, NULL};
 		if ((trg->monitor != NULL && m != trg->monitor) ||
-		    (mon_sel != NULL && !monitor_matches(&loc, ref, mon_sel))) {
+		    (mon_sel != NULL && !monitor_matches(&loc, mon_ref, mon_sel))) {
 			continue;
 		}
 		for (desktop_t *d = m->desk_head; d != NULL; d = d->next) {
@@ -455,10 +455,11 @@ void print_rule_consequence(char **buf, rule_consequence_t *csq)
 		*rect_buf = '\0';
 	}
 
-	asprintf(buf, "monitor=%s desktop=%s node=%s state=%s layer=%s split_dir=%s split_ratio=%lf hidden=%s sticky=%s private=%s locked=%s marked=%s center=%s follow=%s manage=%s focus=%s border=%s rectangle=%s",
+	asprintf(buf, "monitor=%s desktop=%s node=%s state=%s layer=%s honor_size_hints=%s split_dir=%s split_ratio=%lf hidden=%s sticky=%s private=%s locked=%s marked=%s center=%s follow=%s manage=%s focus=%s border=%s rectangle=%s",
 	        csq->monitor_desc, csq->desktop_desc, csq->node_desc,
 	        csq->state == NULL ? "" : STATE_STR(*csq->state),
 	        csq->layer == NULL ? "" : LAYER_STR(*csq->layer),
+	        csq->honor_size_hints == HONOR_SIZE_HINTS_DEFAULT ? "" : HSH_MODE_STR(csq->honor_size_hints),
 	        csq->split_dir == NULL ? "" : SPLIT_DIR_STR(*csq->split_dir), csq->split_ratio,
 	        ON_OFF_STR(csq->hidden), ON_OFF_STR(csq->sticky), ON_OFF_STR(csq->private),
 	        ON_OFF_STR(csq->locked), ON_OFF_STR(csq->marked), ON_OFF_STR(csq->center), ON_OFF_STR(csq->follow),
@@ -511,7 +512,11 @@ desktop_select_t make_desktop_select(void)
 		.focused = OPTION_NONE,
 		.active = OPTION_NONE,
 		.urgent = OPTION_NONE,
-		.local = OPTION_NONE
+		.local = OPTION_NONE,
+		.tiled = OPTION_NONE,
+		.monocle = OPTION_NONE,
+		.user_tiled = OPTION_NONE,
+		.user_monocle = OPTION_NONE
 	};
 	return sel;
 }
@@ -527,6 +532,8 @@ monitor_select_t make_monitor_select(void)
 
 int node_from_desc(char *desc, coordinates_t *ref, coordinates_t *dst)
 {
+	dst->node = NULL;
+
 	coordinates_t ref_copy = *ref;
 	ref = &ref_copy;
 	char *desc_copy = copy_string(desc, strlen(desc));
@@ -536,7 +543,7 @@ int node_from_desc(char *desc, coordinates_t *ref, coordinates_t *dst)
 	char *path = strrchr(desc, '@');
 	char *colon = strrchr(desc, ':');
 
-	/* Discard hashes inside a DESKTOP_SEL */
+	/* Adjust or discard hashes inside a DESKTOP_SEL, e.g. `newest#@prev#older:/1/2` */
 	if (hash != NULL && colon != NULL && path != NULL &&
 	    path < hash && hash < colon) {
 		if (path > desc && *(path - 1) == '#') {
@@ -558,6 +565,11 @@ int node_from_desc(char *desc, coordinates_t *ref, coordinates_t *dst)
 		}
 	}
 
+	/* Discard colons within references, e.g. `@next.occupied:/#any.descendant_of.window` */
+	if (colon != NULL && hash != NULL && colon < hash) {
+		colon = NULL;
+	}
+
 	node_select_t sel = make_node_select();
 
 	if (!parse_node_modifiers(colon != NULL ? colon : desc, &sel)) {
@@ -565,8 +577,6 @@ int node_from_desc(char *desc, coordinates_t *ref, coordinates_t *dst)
 		return SELECTOR_BAD_MODIFIERS;
 	}
 
-	dst->node = NULL;
-
 	direction_t dir;
 	cycle_dir_t cyc;
 	history_dir_t hdi;
@@ -676,6 +686,8 @@ int node_from_desc(char *desc, coordinates_t *ref, coordinates_t *dst)
 
 int desktop_from_desc(char *desc, coordinates_t *ref, coordinates_t *dst)
 {
+	dst->desktop = NULL;
+
 	if (*desc == '%') {
 		locate_desktop(desc + 1, dst);
 		goto end;
@@ -687,6 +699,12 @@ int desktop_from_desc(char *desc, coordinates_t *ref, coordinates_t *dst)
 	desc = desc_copy;
 
 	char *hash = strrchr(desc, '#');
+	char *colon = strrchr(desc, ':');
+
+	/* Discard hashes inside a MONITOR_SEL, e.g. `primary#next:focused` */
+	if (hash != NULL && colon != NULL && hash < colon) {
+		hash = NULL;
+	}
 
 	if (hash != NULL) {
 		*hash = '\0';
@@ -700,16 +718,18 @@ int desktop_from_desc(char *desc, coordinates_t *ref, coordinates_t *dst)
 		}
 	}
 
+	/* Discard colons within references, e.g. `DisplayPort-1:focused#next.local` */
+	if (colon != NULL && hash != NULL && colon < hash) {
+		colon = NULL;
+	}
+
 	desktop_select_t sel = make_desktop_select();
-	char *colon = strrchr(desc, ':');
 
 	if (!parse_desktop_modifiers(colon != NULL ? colon : desc, &sel)) {
 		free(desc_copy);
 		return SELECTOR_BAD_MODIFIERS;
 	}
 
-	dst->desktop = NULL;
-
 	cycle_dir_t cyc;
 	history_dir_t hdi;
 	uint16_t idx;
@@ -794,6 +814,8 @@ end:
 
 int monitor_from_desc(char *desc, coordinates_t *ref, coordinates_t *dst)
 {
+	dst->monitor = NULL;
+
 	if (*desc == '%') {
 		locate_monitor(desc + 1, dst);
 		goto end;
@@ -825,8 +847,6 @@ int monitor_from_desc(char *desc, coordinates_t *ref, coordinates_t *dst)
 		return SELECTOR_BAD_MODIFIERS;
 	}
 
-	dst->monitor = NULL;
-
 	direction_t dir;
 	cycle_dir_t cyc;
 	history_dir_t hdi;
@@ -1116,26 +1136,16 @@ bool node_matches(coordinates_t *loc, coordinates_t *ref, node_select_t *sel)
 	NFLAG(marked)
 #undef NFLAG
 
-	if (loc->node->client == NULL &&
-		(sel->same_class != OPTION_NONE ||
-		 sel->tiled != OPTION_NONE ||
-		 sel->pseudo_tiled != OPTION_NONE ||
-		 sel->floating != OPTION_NONE ||
-		 sel->fullscreen != OPTION_NONE ||
-		 sel->below != OPTION_NONE ||
-		 sel->normal != OPTION_NONE ||
-		 sel->above != OPTION_NONE ||
-		 sel->urgent != OPTION_NONE)) {
-		return false;
-	}
-
-	if (ref->node != NULL && ref->node->client != NULL &&
-	    sel->same_class != OPTION_NONE &&
-	    streq(loc->node->client->class_name, ref->node->client->class_name)
-	    ? sel->same_class == OPTION_FALSE
-	    : sel->same_class == OPTION_TRUE) {
-		return false;
+#define NSPLIT(p, e) \
+	if (sel->p != OPTION_NONE && \
+	    loc->node->split_type != e \
+	    ? sel->p == OPTION_TRUE \
+	    : sel->p == OPTION_FALSE) { \
+		return false; \
 	}
+	NSPLIT(horizontal, TYPE_HORIZONTAL)
+	NSPLIT(vertical, TYPE_VERTICAL)
+#undef NSPLIT
 
 	if (sel->descendant_of != OPTION_NONE &&
 	    !is_descendant(loc->node, ref->node)
@@ -1151,6 +1161,29 @@ bool node_matches(coordinates_t *loc, coordinates_t *ref, node_select_t *sel)
 		return false;
 	}
 
+	if (loc->node->client == NULL) {
+		if (sel->same_class == OPTION_TRUE ||
+		    sel->tiled == OPTION_TRUE ||
+		    sel->pseudo_tiled == OPTION_TRUE ||
+		    sel->floating == OPTION_TRUE ||
+		    sel->fullscreen == OPTION_TRUE ||
+		    sel->below == OPTION_TRUE ||
+		    sel->normal == OPTION_TRUE ||
+		    sel->above == OPTION_TRUE ||
+		    sel->urgent == OPTION_TRUE) {
+			return false;
+		}
+		return true;
+	}
+
+	if (ref->node != NULL && ref->node->client != NULL &&
+	    sel->same_class != OPTION_NONE &&
+	    streq(loc->node->client->class_name, ref->node->client->class_name)
+	    ? sel->same_class == OPTION_FALSE
+	    : sel->same_class == OPTION_TRUE) {
+		return false;
+	}
+
 #define WSTATE(p, e) \
 	if (sel->p != OPTION_NONE && \
 	    loc->node->client->state != e \
@@ -1186,20 +1219,6 @@ bool node_matches(coordinates_t *loc, coordinates_t *ref, node_select_t *sel)
 	WFLAG(urgent)
 #undef WFLAG
 
-	if (sel->horizontal != OPTION_NONE &&
-	    loc->node->split_type != TYPE_HORIZONTAL
-	    ? sel->horizontal == OPTION_TRUE
-	    : sel->horizontal == OPTION_FALSE) {
-		return false;
-	}
-
-	if (sel->vertical != OPTION_NONE &&
-	    loc->node->split_type != TYPE_VERTICAL
-	    ? sel->vertical == OPTION_TRUE
-	    : sel->vertical == OPTION_FALSE) {
-		return false;
-	}
-
 	return true;
 }
 
@@ -1240,6 +1259,28 @@ bool desktop_matches(coordinates_t *loc, coordinates_t *ref, desktop_select_t *s
 		return false;
 	}
 
+#define DLAYOUT(p, e) \
+	if (sel->p != OPTION_NONE && \
+	    loc->desktop->layout != e \
+	    ? sel->p == OPTION_TRUE \
+	    : sel->p == OPTION_FALSE) { \
+		return false; \
+	}
+	DLAYOUT(tiled, LAYOUT_TILED)
+	DLAYOUT(monocle, LAYOUT_MONOCLE)
+#undef DLAYOUT
+
+#define DUSERLAYOUT(p, e) \
+	if (sel->p != OPTION_NONE && \
+	    loc->desktop->user_layout != e \
+	    ? sel->p == OPTION_TRUE \
+	    : sel->p == OPTION_FALSE) { \
+		return false; \
+	}
+	DUSERLAYOUT(user_tiled, LAYOUT_TILED)
+	DUSERLAYOUT(user_monocle, LAYOUT_MONOCLE)
+#undef DUSERLAYOUT
+
 	return true;
 }
 
